/*____________________________________________________________________________
	
	Zinf - Zinf Is Not FreeA*p (The Free MP3 Player)

	Portions Copyright (C) 1999 EMusic.com

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
	
	$Id: id3lib.cpp,v 1.4 2003/02/13 12:11:35 rottmann Exp $
____________________________________________________________________________*/

#include <stdio.h>
#include <string>
#include <stdlib.h>
#include <assert.h>
#include <errno.h>

#ifdef __QNX__
#include <strings.h>
#endif
using namespace std;
#include "errors.h"
#include "utility.h"
#include "debug.h"

#include "id3lib.h"

#include <id3/tag.h>


#define DB printf("%s:%d\n", __FILE__, __LINE__);

const int iDataFieldLen = 1024;
const int genreOther = 12;
#define genreMax (int)(sizeof(genreList)/sizeof(char*))

const char *genreList[] = 
{
     "Blues",
     "Classic Rock",
     "Country",
     "Dance",
     "Disco",
     "Funk",
     "Grunge",
     "Hip-Hop",
     "Jazz",
     "Metal",
     "New Age",
     "Oldies",
     "Other",
     "Pop",
     "R&B",
     "Rap",
     "Reggae",
     "Rock",
     "Techno",
     "Industrial",
     "Alternative",
     "Ska",
     "Death Metal",
     "Pranks",
     "Soundtrack",
     "Euro-Techno",
     "Ambient",
     "Trip-Hop",
     "Vocal",
     "Jazz+Funk",
     "Fusion",
     "Trance",
     "Classical",
     "Instrumental",
     "Acid",
     "House",
     "Game",
     "Sound Clip",
     "Gospel",
     "Noise",
     "AlternRock",
     "Bass",
     "Soul",
     "Punk",
     "Space",
     "Meditative",
     "Instrumental Pop",
     "Instrumental Rock",
     "Ethnic",
     "Gothic",
     "Darkwave",
     "Techno-Industrial",
     "Electronic",
     "Pop-Folk",
     "Eurodance",
     "Dream",
     "Southern Rock",
     "Comedy",
     "Cult",
     "Gangsta",
     "Top 40",
     "Christian Rap",
     "Pop/Funk",
     "Jungle",
     "Native American",
     "Cabaret",
     "New Wave",
     "Psychadelic",
     "Rave",
     "Showtunes",
     "Trailer",
     "Lo-Fi",
     "Tribal",
     "Acid Punk",
     "Acid Jazz",
     "Polka",
     "Retro",
     "Musical",
     "Rock & Roll",
     "Hard Rock",
     "Folk",
     "Folk-Rock",
     "National Folk",
     "Swing",
     "Fast Fusion",
     "Bebob",
     "Latin",
     "Revival",
     "Celtic",
     "Bluegrass",
     "Avantgarde",
     "Gothic Rock",
     "Progressive Rock",
     "Psychedelic Rock",
     "Symphonic Rock",
     "Slow Rock",
     "Big Band",
     "Chorus",
     "Easy Listening",
     "Acoustic",
     "Humour",
     "Speech",
     "Chanson",
     "Opera",
     "Chamber Music",
     "Sonata",
     "Symphony",
     "Booty Bass",
     "Primus",
     "Porn Groove",
     "Satire",
     "Slow Jam",
     "Club",
     "Tango",
     "Samba",
     "Folklore",
     "Ballad",
     "Power Ballad",
     "Rhythmic Soul",
     "Freestyle",
     "Duet",
     "Punk Rock",
     "Drum Solo",
     "Acapella",
     "Euro-House",
     "Dance Hall",
};

#ifndef COMPILING_SIGAPP
extern "C"
{
   MetaDataFormat *Initialize(FAContext* context)
   {
      return new ID3lib(context);
   }
}
#endif

ID3lib::ID3lib(FAContext* context)
  : MetaDataFormat(context)
{
}

ID3lib::~ID3lib()
{
}


static
bool getTag(ID3_Tag&tag, ID3_FrameID frameid, string &result)
{
    static char buffer[1024];

    ID3_Frame *frame;
    ID3_Field *field;

    frame = tag.Find(frameid);
    if (frame) {
      field = frame->GetField (ID3FN_TEXT);
      if (field) 
        if (field->Get (buffer, sizeof buffer) > 0) {
          result = buffer;
          return true;
        }
    }
    return false;
}

static
bool setTag(ID3_Tag&tag, ID3_FrameID frameid, const string &result)
{
  
    ID3_Frame *frame;
    ID3_Field *field;

    frame = tag.Find(frameid);
    if (!frame) { 
      frame = new ID3_Frame(frameid); 
      tag.AttachFrame (frame);
    }
    field = frame->GetField (ID3FN_TEXT);
    field->Set (result.c_str());

    return true;
}

bool ID3lib::ReadMetaData(const char* url, MetaData* metadata)
{
    assert(url);
    assert(metadata);

    char *ptr = strrchr(url, '.');
    if (ptr == NULL)
        return false;

    if (strcasecmp(ptr, ".mp3")) 
        return false;

    string path;
    URLToFilePath(url, path);

    ID3_Tag tag;
    string val;
    tag.Link( path.c_str());

    if (getTag(tag, ID3FID_TITLE,  val)) 
      metadata->SetTitle(val);
    if (getTag(tag, ID3FID_ALBUM,  val)) 
      metadata->SetAlbum(val);
    if (getTag(tag, ID3FID_LEADARTIST,  val)) 
      metadata->SetArtist(val);
    else if (getTag(tag, ID3FID_BAND,  val)) 
      metadata->SetArtist(val);
    if (getTag(tag, ID3FID_COMMENT,  val)) 
      metadata->SetComment(val);
    if (getTag(tag, ID3FID_SONGLEN,  val)) 
      metadata->SetTime(atoi(val.c_str()) / 1000);
    if (getTag(tag, ID3FID_YEAR,  val)) 
      metadata->SetYear(atoi(val.c_str()));
    if (getTag(tag, ID3FID_SIZE,  val)) 
      metadata->SetSize(atoi(val.c_str()));
    if (getTag(tag, ID3FID_TRACKNUM,  val)) 
      metadata->SetTrack(atoi(val.c_str()));
    if (getTag(tag, ID3FID_CONTENTTYPE,  val)) {
        char genre[255];
        int  genreId, ret;
        ret = sscanf(val.c_str(), "(%d)%[^\n]", &genreId, genre);
        if (ret > 0) {
          if (ret == 2)
            metadata->SetGenre(genre);
          else
            if (genreId != genreOther) {
              LookupGenre(genreId, genre);
              metadata->SetGenre(genre);
            }
        }
        else {
          metadata->SetGenre(val);
        }
    }

    return true;
}

bool ID3lib::WriteMetaData(const char* url, const MetaData& metadata)
{
    char      dummy[20];
    bool      bWriteID3v1, bWriteID3v2;
    luint     whichTags;
    char     *ptr;

    ptr = strrchr(url, '.');
    if (ptr == NULL)
        return false;

    if (strcasecmp(ptr, ".mp3"))
        return false;  

    m_context->prefs->GetPrefBoolean(kWriteID3v1Pref, &bWriteID3v1);
    m_context->prefs->GetPrefBoolean(kWriteID3v2Pref, &bWriteID3v2);

    whichTags = (bWriteID3v1) ? ID3TT_ID3V1 : 0;
    whichTags |= (bWriteID3v2) ? ID3TT_ID3V2 : 0;

    if (!whichTags)
        return true;

    assert(url);

    string path;
    URLToFilePath(url, path);

    ID3_Tag tag;

    tag.Link (path.c_str(), whichTags);

    setTag(tag, ID3FID_TITLE, metadata.Title());
    setTag(tag, ID3FID_ALBUM, metadata.Album());
    setTag(tag, ID3FID_LEADARTIST, metadata.Artist());
    setTag(tag, ID3FID_COMMENT, metadata.Comment());

    sprintf(dummy, "%d", metadata.Time() * 1000);
    setTag(tag, ID3FID_SONGLEN, dummy);

    sprintf(dummy, "%d", metadata.Year());
    setTag(tag, ID3FID_YEAR, dummy);

    sprintf(dummy, "%d", metadata.Size());
    setTag(tag, ID3FID_SIZE, dummy);

    sprintf(dummy, "%d", metadata.Track());
    setTag(tag, ID3FID_TRACKNUM, dummy);

    sprintf(dummy, "(12)%s", metadata.Genre().c_str());
    setTag(tag, ID3FID_CONTENTTYPE, dummy);

    tag.Update();
    return true;
}


void ID3lib::LookupGenre(int genreId, char* genre)
{
    if (genreId < 0 || genreId > genreMax)
    {
        genre[0] = 0;
        return;
    }

    strcpy(genre, genreList[genreId]);
}

